Documentation
¶
Overview ¶
Package ebnf parses extended Backus-Naur Form (EBNF) grammars.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type Grammar ¶
type Grammar struct {
Prods []*Prod
}
Grammar is an abstract syntax tree for a grammar.
type ParserError ¶
type ParserError struct {
Errs []error
}
ParserError has all the parse errors.
func (ParserError) Error ¶
func (e ParserError) Error() string
Error returns all the error strings joined by a newline.
Click to show internal directories.
Click to hide internal directories.